package com.hisense.pangea.preview.demo;

import com.hisense.pangea.exception.BaseException;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections.MapUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

import static org.apache.commons.lang3.StringUtils.isAnyEmpty;

/**
 * 签名生成工具类
 */
public class SignatureUtils {

    /**
     * 生成常规请求签名
     *
     * @param secretKey 密钥
     * @param method    请求方法（GET/POST等）
     * @param urlParams URL参数
     * @param body      请求体内容
     * @return MD5签名
     */
    public static String generateSign(String secretKey, String method, Map<String, String> urlParams, String body) throws IOException {
        validateSign(secretKey, method);
        TreeMap<String, String> queryMap = mergeMapWithListValue(urlParams);
        String query = buildQueryString(queryMap);
        String sign = buildSignParameters(method, query, body, secretKey, null, null);
        return DigestUtils.md5Hex(sign);
    }

    /**
     * 生成文件上传请求签名
     *
     * @param secretKey 密钥
     * @param method    请求方法
     * @param urlParams URL参数
     * @param file      上传文件
     * @return MD5签名
     */
    public static String generateSign(String secretKey, String method, Map<String, String> urlParams, MultipartFile file) throws IOException {
        validateSign(secretKey, method);
        TreeMap<String, String> queryMap = mergeMapWithListValue(urlParams);
        String query = buildQueryString(queryMap);
        //method=POST&body.file=eb80571d7f3fe10cc8a6a41e809dc5e4&salt=514A7962EE3749FE86E911BA007E8DDC"
        //method=POST&body.file=eb80571d7f3fe10cc8a6a41e809dc5e4&salt=f525f567c3fbf0bac95d12231cd36f04"
        String sign = buildSignParameters(method, query, null, secretKey, file, "2");
        return DigestUtils.md5Hex(sign);
    }

    /**
     * 验证时间戳有效性（±5分钟）
     *
     * @param timestampStr 毫秒级时间戳
     * @throws BaseException 验证失败时抛出
     */
    public static void validateTimestamp(String timestampStr) throws BaseException {
        if (isAnyEmpty(timestampStr)) {
            throw new BaseException("400", "时间戳不能为空");
        }

        try {
            long timestamp = Long.parseLong(timestampStr);
            long currentTime = System.currentTimeMillis();
            long fiveMinutes = 5 * 60 * 1000;

            if (Math.abs(currentTime - timestamp) > fiveMinutes) {
                throw new BaseException("403", "请求已超时");
            }
        } catch (NumberFormatException e) {
            throw new BaseException("400", "时间戳格式错误");
        }
    }

    // ========================= 私有方法 =========================

    private static String buildQueryString(Map<String, String> queryMap) {
        return queryMap.entrySet().stream()
                .map(e -> "query." + e.getKey() + "=" + e.getValue())
                .collect(Collectors.joining("&"));
    }

    private static TreeMap<String, String> mergeMapWithListValue(Map<String, String> sourceMap) {
        return MapUtils.isEmpty(sourceMap) ? new TreeMap<>() : new TreeMap<>(sourceMap);
    }

    private static String buildSignParameters(String method, String query, String body, String secretKey, MultipartFile file,String fileSignMode) throws IOException {
        StringBuilder sb = new StringBuilder(256);
        sb.append("method=").append(method.toUpperCase()).append("&");

        if (!isAnyEmpty(query)) {
            sb.append(query).append("&");
        }
        if (!isAnyEmpty(body)) {
            sb.append("body.").append(body.replaceAll("\\s", "")).append("&");
        }
        if (file != null) {
            //文件加密方式默认为1，1：文件名+文件大小，2：文件MD5
            if (fileSignMode.equals("1")) {
                sb.append("body.file=[name=")
                        .append(file.getOriginalFilename())
                        .append("&size=")
                        .append(file.getSize())
                        .append("]&");
            } else {
                // body.file=eb80571d7f3fe10cc8a6a41e809dc5e4
                String fileMd5 = getFileMd5(file);
                sb.append("body.file=")
                        .append(fileMd5)
                        .append("&");
            }
        }

        sb.append("salt=").append(secretKey);
        return sb.toString();
    }

    /**
     * 验证签名
     * @param secretKey 签名密钥
     * @param method    方法类型
     */
    private static void validateSign(String secretKey, String method) {
        if (isAnyEmpty(secretKey, method)) {
            throw new BaseException("400", "生成签名字段不能为空: secretKey, method");
        }
    }

    /**
     *
     * @param file 文件
     * @return 加密串
     */
    public static String getFileMd5(MultipartFile file) throws IOException {
        try (InputStream inputStream = file.getInputStream()) {
            return DigestUtils.md5Hex(inputStream); // 直接对文件流进行MD5加密
        }
    }
}
